<?php

defined( 'ABSPATH' ) || exit;

require_once PCNCH_PLUGIN_PATH . 'admin/admin.php';

/**
 * Registers a submenu page for the News Communications Hub under the main menu 'pragmaticcoders'.
 *
 * @return void
 */
function pcnch_menu() {
    add_submenu_page(
        'pragmaticcoders',
        'News Communications Hub',
        'News Communications Hub',
        'manage_options',
        'pragmaticcoders-nch',
        'pcnch_page'
    );
}
add_action('admin_menu', 'pcnch_menu');

/**
 * Enqueues the JavaScript file for the admin area of the plugin.
 *
 * @return void
 */
function pcnch_enqueue_admin_script() {
    wp_enqueue_script(
        'nch-script-admin',
        PCNCH_PLUGIN_URL . 'assets/js/nch-script-admin.js',
        array('jquery'),
        '1.0.0',
        true
    );
}
add_action('admin_enqueue_scripts', 'pcnch_enqueue_admin_script');

/**
 * Enqueues the Dashicons style if it's not already enqueued.
 *
 * @return void
 */
function pcnch_enqueue_dashicons_if_needed() {
    if (!wp_style_is('dashicons', 'enqueued')) {
        wp_enqueue_style('dashicons', includes_url('css/dashicons.min.css'), array(), get_bloginfo('version'), 'all');
    }
}
add_action('wp_enqueue_scripts', 'pcnch_enqueue_dashicons_if_needed');

/**
 * Returns an array of allowed HTML tags with additional attributes for <input> and <div>.
 *
 * @return array The allowed HTML tags with their respective attributes.
 */
function pcnch_kses_allowed_tags() {
    $allowed_tags = wp_kses_allowed_html('post');

    $allowed_tags['input'] = [
        'type'  => [],
        'name'  => [],
        'value' => [],
        'class' => [],
        'style' => [],
    ];
    $allowed_tags['div']['style'] = [];

    return $allowed_tags;
}

/**
 * Sanitize a value to an integer.
 *
 * @param mixed $value The value to sanitize.
 * @return int Sanitized integer value.
 */
function pcnch_sanitize_number($value) {
    return intval($value);
}

/**
 * Sanitize an array of post types.
 *
 * @param mixed $value The value to sanitize, expected as an array or single value.
 * @return array Sanitized post types.
 */
function pcnch_sanitize_post_types($value) {
    return array_map('sanitize_key', (array) $value);
}

/**
 * Sanitize post icons option.
 *
 * @param array $input Input data to sanitize.
 * @return array Sanitized data.
 */
function pcnch_sanitize_post_icons($input) {
    $sanitized = array();

    if (is_array($input)) {
        foreach ($input as $key => $value) {
            $sanitized[sanitize_text_field($key)] = sanitize_text_field($value);
        }
    }

    return $sanitized;
}

/**
 * Sanitize a checkbox value (1 or 0).
 *
 * @param mixed $value The value to sanitize.
 * @return int 1 if the value is equal to 1, otherwise 0.
 */
function pcnch_sanitize_checkbox($value) {
    return (intval($value) === 1) ? 1 : 0;
}

/**
 * Sanitize a custom message with an icon and text.
 *
 * @param string $value The raw message to sanitize.
 * @return string Sanitized message or an empty string if invalid.
 */
function pcnch_sanitize_message($value) {
    if (preg_match('/<i class="(.+?)"><\/i>\s*<span>(.+?)<\/span>/', $value, $matches)) {
        $icon_class = sanitize_text_field($matches[1]);
        $text_content = sanitize_text_field($matches[2]);
        return sprintf('<i class="%s"></i> <span>%s</span>', esc_attr($icon_class), esc_html($text_content));
    }
    return '';
}

/**
 * Sanitize an array of custom notifications.
 *
 * @param mixed $notifications The notifications to sanitize.
 * @return array Sanitized notifications.
 */
function pcnch_sanitize_custom_notifications($notifications) {
    if (!is_array($notifications)) {
        return [];
    }

    return array_map(function ($notification) {
        $notification['id'] = isset($notification['id']) ? sanitize_text_field($notification['id']) : uniqid();
        $notification['date'] = isset($notification['date']) ? sanitize_text_field($notification['date']) : current_time('Y-m-d\TH:i');
        $notification['date_to'] = isset($notification['date_to']) ? sanitize_text_field($notification['date_to']) : '';
        $notification['show_on_first_visit'] = isset($notification['show_on_first_visit']) ? (bool) $notification['show_on_first_visit'] : false;
        $notification['show_on_top'] = isset($notification['show_on_top']) ? (bool) $notification['show_on_top'] : false;
        $notification['icon'] = isset($notification['icon']) ? sanitize_text_field($notification['icon']) : 'dashicons dashicons-book';
        $notification['tags'] = isset($notification['tags']) ? implode(', ', array_map('sanitize_text_field', (array) $notification['tags'])) : '';
        $notification['color'] = isset($notification['color']) ? sanitize_hex_color($notification['color']) : '#00a571';
        return $notification;
    }, $notifications);
}

/**
 * Renders the main settings page for the News Communications Hub plugin.
 *
 * @return void
 */
function pcnch_page() {

    if (!current_user_can('manage_options')) {
        return;
    }

    $is_log_enabled = get_option('pcnch_enable_log', 0);
    $active_tab = isset($_GET['tab']) ? sanitize_text_field(wp_unslash($_GET['tab'])) : 'custom_notifications';
    ?>
    <div class="wrap pragmaticcoders-admin">
        <h1><?php esc_attr_e('News Communications Hub', 'news-communications-hub'); ?> <small><?php echo esc_html( PCNCH_VERSION ); ?></small></h1>
        <h2 class="nav-tab-wrapper">
            <a href="?page=pragmaticcoders-nch&tab=custom_notifications" class="nav-tab <?php echo $active_tab === 'custom_notifications' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('Custom Notifications', 'news-communications-hub'); ?></a>
            <a href="?page=pragmaticcoders-nch&tab=general" class="nav-tab <?php echo $active_tab === 'general' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('Settings', 'news-communications-hub'); ?></a>
            <a href="?page=pragmaticcoders-nch&tab=cron" class="nav-tab <?php echo $active_tab === 'cron' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('CRON Settings', 'news-communications-hub'); ?></a>
            <a href="?page=pragmaticcoders-nch&tab=transients" class="nav-tab <?php echo $active_tab === 'transients' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('Transients', 'news-communications-hub'); ?></a>
        <?php if ($is_log_enabled): ?>
            <a href="?page=pragmaticcoders-nch&tab=log" class="nav-tab <?php echo $active_tab === 'log' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('Log', 'news-communications-hub'); ?></a>
        <?php endif; ?>
            <a href="?page=pragmaticcoders-nch&tab=about" class="nav-tab <?php echo $active_tab === 'about' ? 'nav-tab-active' : ''; ?>"><?php esc_html_e('About', 'news-communications-hub'); ?></a>
       
        </h2>

        <div class="tab-content">
            <?php
            switch ($active_tab) {
                case 'general':
                    echo '<form method="post" action="options.php">';
                    settings_fields('pcnch_general_group');
                    do_settings_sections('pragmaticcoders-nch-general');
                    submit_button();
                    echo '</form>';
                    break;

                case 'cron':
                    echo '<form method="post" action="options.php">';
                    settings_fields('pcnch_cron_group');
                    do_settings_sections('pragmaticcoders-nch-cron');
                    submit_button();
                    echo '</form>';
                    break;

                case 'transients':
                    $daily_created_at = pcnch_get_transient_created_at('pcnch_daily_notifications');
                    $first_visit_created_at = pcnch_get_transient_created_at('pcnch_first_visit_notifications');
                    ?>
                    <h3><?php esc_html_e('Manage Transients', 'news-communications-hub'); ?></h3>
                    
                    <form method="post" action="">
                        <p><?php esc_html_e('Regenerate transients for first visit and daily.', 'news-communications-hub'); ?></p>
                        <input type="hidden" name="regenerate_transients" value="1" />
                        <?php submit_button(esc_html__('Regenerate Transients', 'news-communications-hub'), 'primary'); ?>
                    </form>

                    <table class="widefat fixed" cellspacing="0">
                        <thead>
                            <tr>
                                <th><?php esc_html_e('Transient Name', 'news-communications-hub'); ?></th>
                                <th><?php esc_html_e('Created At', 'news-communications-hub'); ?></th>
                            </tr>
                        </thead>
                        <tbody>
                            <tr>
                                <td><?php esc_html_e('Daily Notifications', 'news-communications-hub'); ?></td>
                                <td><?php echo $daily_created_at ? esc_html($daily_created_at) : esc_html__('Not available', 'news-communications-hub'); ?></td>
                            </tr>
                            <tr>
                                <td><?php esc_html_e('First Visit Notifications', 'news-communications-hub'); ?></td>
                                <td><?php echo $first_visit_created_at ? esc_html($first_visit_created_at) : esc_html__('Not available', 'news-communications-hub'); ?></td>
                            </tr>
                        </tbody>
                    </table>
                  

                    <?php
                    if (!current_user_can('manage_options')) {
                        return;
                    }

                    if (isset($_POST['regenerate_transients'])) {
                        delete_transient('pcnch_first_visit_notifications');
                        delete_transient('pcnch_daily_notifications');
                        pcnch_generate_transients();
                        pcnch_error_log("--# TRANSIENT DATA REFRESHED MANUALLY #--");
                        echo '<div class="notice notice-success is-dismissible"><p>Transients have been regenerated.</p></div>';
                    }
                    break;


                case 'log':
                    if ($is_log_enabled) {
                        echo '<h2>' . esc_html__('Log', 'news-communications-hub') . '</h2>';
                        pcnch_display_log();
                    }
                    break;

                case 'about':
                    ?>
                    <?php
                        pcnch_about_page();
                    break;

                case 'custom_notifications':
                    default:
                    echo '<form method="post" action="options.php">';
                    settings_fields('pcnch_custom_notifications_group');
                    do_settings_sections('pragmaticcoders-nch-custom_notifications');
                    submit_button();
                    echo '</form>';
                    break;
            }
            ?>
        </div>
    </div>
    <?php
}


/**
 * Registers settings for different sections including general, custom notifications, CRON settings, and appearance options.
 *
 * @return void
 */
function pcnch_register_settings() {
    // Register
    register_setting('pcnch_general_group', 'pcnch_post_types', 'pcnch_sanitize_post_types');
    register_setting('pcnch_general_group', 'pcnch_update_types', 'pcnch_sanitize_post_types');
    register_setting('pcnch_general_group', 'pcnch_display_limit', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_first_visit_display_limit', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_first_visit_post_types', 'pcnch_sanitize_post_types');
    register_setting('pcnch_general_group', 'pcnch_localstorage_lifetime', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_localstorage_first_visit_lifetime', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_transient_key_lifetime', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_transient_key_first_visit_lifetime', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_post_icons', 'pcnch_sanitize_post_icons');
    register_setting('pcnch_general_group', 'pcnch_list_position', 'sanitize_text_field');
    register_setting('pcnch_general_group', 'pcnch_list_width', 'pcnch_sanitize_number');
    register_setting('pcnch_general_group', 'pcnch_enable_log', 'pcnch_sanitize_checkbox');
    register_setting('pcnch_general_group', 'pcnch_read_icon', 'sanitize_text_field');
    register_setting('pcnch_general_group', 'pcnch_show_logo', 'pcnch_sanitize_checkbox');
    register_setting('pcnch_general_group', 'pcnch_mark_all_as_read', 'pcnch_sanitize_message');
    register_setting('pcnch_general_group', 'pcnch_show_read_items', 'pcnch_sanitize_message');
    register_setting('pcnch_general_group', 'pcnch_hide_read_items', 'pcnch_sanitize_message');
    register_setting('pcnch_general_group', 'pcnch_all_read_message', 'pcnch_sanitize_message');

    // Custom Notifications group
    register_setting('pcnch_custom_notifications_group', 'pcnch_custom_notifications', 'pcnch_sanitize_custom_notifications');

    // CRON group
    register_setting('pcnch_cron_group', 'pcnch_cron_frequency', 'sanitize_text_field');
    register_setting('pcnch_cron_group', 'pcnch_cron_time', 'sanitize_text_field');

    // Section General
    add_settings_section('pcnch_general_section', __('General Settings', 'news-communications-hub'), null, 'pragmaticcoders-nch-general');
    add_settings_field('pcnch_update_types', __('Update types to track', 'news-communications-hub'), 'pcnch_update_types_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_post_types', __('Post types to track', 'news-communications-hub'), 'pcnch_post_types_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_first_visit_post_types', __('Post types to track for first visit', 'news-communications-hub'), 'pcnch_first_visit_post_types_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_display_limit', __('Number of notifications', 'news-communications-hub'), 'pcnch_display_limit_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_first_visit_display_limit', __('Number of notifications on first visit', 'news-communications-hub'), 'pcnch_first_visit_display_limit_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_localstorage_lifetime', __('LocalStorage Lifetime', 'news-communications-hub'), 'pcnch_localstorage_lifetime_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_localstorage_first_visit_lifetime', __('First visit LocalStorage Lifetime', 'news-communications-hub'), 'pcnch_localstorage_first_visit_lifetime_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_transient_key_lifetime', __('Transient Key Lifetime', 'news-communications-hub'), 'pcnch_transient_key_lifetime_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_transient_key_first_visit_lifetime', __('First visit Transient Key Lifetime', 'news-communications-hub'), 'pcnch_transient_key_first_visit_lifetime_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_field('pcnch_enable_log', __('Enable Log', 'news-communications-hub'), 'pcnch_enable_log_callback', 'pragmaticcoders-nch-general', 'pcnch_general_section');
    add_settings_section('pcnch_appearance_section', __('Appearance', 'news-communications-hub'), null, 'pragmaticcoders-nch-general');
    add_settings_field('pcnch_list_position', __('List position', 'news-communications-hub'), 'pcnch_list_position_callback', 'pragmaticcoders-nch-general', 'pcnch_appearance_section');
    add_settings_field('pcnch_list_width', __('List width (px)', 'news-communications-hub'), 'pcnch_list_width_callback', 'pragmaticcoders-nch-general', 'pcnch_appearance_section');
    add_settings_section('pcnch_icons_section', __('Icons', 'news-communications-hub'), null, 'pragmaticcoders-nch-general');
    add_settings_field('pcnch_post_icons', __('Post Type Icons', 'news-communications-hub'), 'pcnch_post_icons_callback', 'pragmaticcoders-nch-general', 'pcnch_icons_section');
    add_settings_section( 'pcnch_messages_section', __('Button Labels and Messages', 'news-communications-hub'), null, 'pragmaticcoders-nch-general' );
    add_settings_field( 'pcnch_read_icon', __('Read icon', 'news-communications-hub'), 'pcnch_read_icon_callback', 'pragmaticcoders-nch-general', 'pcnch_messages_section' );
    add_settings_field( 'pcnch_mark_all_as_read', __('Mark All as Read Button', 'news-communications-hub'), 'pcnch_mark_all_as_read_callback', 'pragmaticcoders-nch-general', 'pcnch_messages_section' );
    add_settings_field( 'pcnch_show_read_items', __('Show Read Items Button', 'news-communications-hub'), 'pcnch_show_read_items_callback', 'pragmaticcoders-nch-general', 'pcnch_messages_section' );
    add_settings_field( 'pcnch_hide_read_items', __('Hide Read Items Button', 'news-communications-hub'), 'pcnch_hide_read_items_callback', 'pragmaticcoders-nch-general', 'pcnch_messages_section' );
    add_settings_field( 'pcnch_all_read_message', __('All notifications are read message', 'news-communications-hub'), 'pcnch_all_read_message_callback', 'pragmaticcoders-nch-general', 'pcnch_messages_section' );

    // Section Custom Notifications
    add_settings_section('pcnch_custom_notifications_section', __('Custom Notifications', 'news-communications-hub'), null, 'pragmaticcoders-nch-custom_notifications');
    add_settings_field('pcnch_custom_notifications', __('Notifications', 'news-communications-hub'), 'pcnch_custom_notifications_callback', 'pragmaticcoders-nch-custom_notifications', 'pcnch_custom_notifications_section');

    // Section CRON
    add_settings_section('pcnch_cron_section', __('CRON Settings', 'news-communications-hub'), null, 'pragmaticcoders-nch-cron');
    add_settings_field('pcnch_cron_frequency', __('Transient CRON Frequency', 'news-communications-hub'), 'pcnch_cron_frequency_callback', 'pragmaticcoders-nch-cron', 'pcnch_cron_section');
    add_settings_field('pcnch_cron_time', __('Transient CRON Time', 'news-communications-hub'), 'pcnch_cron_time_callback', 'pragmaticcoders-nch-cron', 'pcnch_cron_section');

}
add_action('admin_init', 'pcnch_register_settings');

/**
 * Displays a list of public post types as checkboxes for selection.
 *
 * @return void
 */
function pcnch_post_types_callback() {
    $post_types = get_option('pcnch_post_types', array());
    if (!is_array($post_types)) {
        $post_types = array(); 
    }

    $all_post_types = get_post_types(array('public' => true), 'objects');
    ?>
    <fieldset>
        <div class="admin-grid">
        <div class="row">
        <?php foreach ($all_post_types as $post_type): ?>
            <div class="col column-3 column-md-6">
            <label>
                <input type="checkbox" name="pcnch_post_types[]" value="<?php echo esc_attr($post_type->name); ?>" <?php checked(in_array($post_type->name, $post_types)); ?>>
                <?php echo esc_html($post_type->label); ?>
            </label>
            </div>
        <?php endforeach; ?>
        </div>
        </div>
    </fieldset>
    <?php
}

/**
 * Displays checkboxes for selecting update types (new and modified posts).
 *
 * @return void
 */
function pcnch_update_types_callback() {
    $update_types = get_option('pcnch_update_types', array('new', 'modified'));
    if (!is_array($update_types)) {
        $update_types = array();
    }    ?>
    <fieldset>
        <label>
            <input type="checkbox" name="pcnch_update_types[]" value="new" <?php checked(in_array('new', $update_types)); ?>>
            <?php esc_attr_e('New Posts', 'news-communications-hub'); ?>
        </label><br>
        <label>
            <input type="checkbox" name="pcnch_update_types[]" value="modified" <?php checked(in_array('modified', $update_types)); ?>>
            <?php esc_attr_e('Modified Posts', 'news-communications-hub'); ?>
        </label>
    </fieldset>
    <?php
}

/**
 * Displays a number input field for setting the display limit of notifications.
 *
 * @return void
 */
function pcnch_display_limit_callback() {
    $display_limit = get_option('pcnch_display_limit', 5);
    ?>
    <input type="number" name="pcnch_display_limit" value="<?php echo esc_attr($display_limit); ?>" min="1" max="30" />
    <p class="description"><?php esc_attr_e('Enter the max. number of notifications to display (1-30).', 'news-communications-hub'); ?></p>
    <?php
}

/**
 * Displays a number input field for setting the display limit of notifications on first visit.
 *
 * @return void
 */
function pcnch_first_visit_display_limit_callback() {
    $pcnch_first_visit_display_limit = get_option('pcnch_first_visit_display_limit', 5);
    ?>
    <input type="number" name="pcnch_first_visit_display_limit" value="<?php echo esc_attr($pcnch_first_visit_display_limit); ?>" min="1" max="30" />
    <p class="description"><?php esc_attr_e('Enter the max. number of notifications to display on first visit (1-30).', 'news-communications-hub') ?></p>
    <?php
}

/**
 * Displays a list of public post types as checkboxes for selection on first visit.
 * 
 * @return void
 */
function pcnch_first_visit_post_types_callback() {
    $first_visit_post_types = get_option('pcnch_first_visit_post_types', array());
    if (!is_array($first_visit_post_types)) {
        $first_visit_post_types = array();
    }
    $all_post_types = get_post_types(array('public' => true), 'objects');
    ?>
    <fieldset>
    <div class="admin-grid">
    <div class="row">
        <?php foreach ($all_post_types as $post_type): ?>
            <div class="col column-3 column-md-6">
                <label>
                <input type="checkbox" name="pcnch_first_visit_post_types[]" value="<?php echo esc_attr($post_type->name); ?>" <?php checked(in_array($post_type->name, $first_visit_post_types)); ?>>
                <?php echo esc_html($post_type->label); ?>
                </label>
            </div>
        <?php endforeach; ?>
    </div>
    </div>
    </fieldset>
    <?php
}

/**
 * Displays a form for managing custom notifications.
 * 
 * @return void
 */
function pcnch_custom_notifications_callback() {
    $notifications = get_option('pcnch_custom_notifications', array());
    $icons = is_plugin_active('elementor/elementor.php') ? pcnch_get_elementor_icons() : array();

    ?>
    <div id="nch-notifications-container">
    <div id="admin-grid">
        <?php
        $custom_notification_index = 0;
        if ($notifications) {    
            foreach ($notifications as $index => $notification) {
                if (!isset($notification['id'])) {
                    $notification['id'] = uniqid();
                }

                $custom_notification_index++;
                $date_value = $notification['date'];
                $date_to_value = $notification['date_to'];
                $show_on_first_visit_value = $notification['show_on_first_visit'];
                $show_on_top_value = $notification['show_on_top'];
                $icon_value = isset($notification['icon']) ? $notification['icon'] : 'dashicons dashicons-book';
                $tags_value = $notification['tags'];
                $color_value = isset($notification['color']) ? $notification['color'] : '#00a571';


                ?>
                <div class="nch-notification" style="margin-bottom: 15px; padding-bottom: 15px; border-bottom: 1px solid #ccc;">
                    <input type="hidden" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][id]" value="<?php echo esc_attr($notification['id']); ?>" />
                    
                    <div class="row">
                    <div class="col column-10 column-md-11">
                        <div class="row">

                        <div class="col column-4 column-md-6">
                            <input type="text" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][title]" value="<?php echo esc_attr($notification['title']); ?>" placeholder="<?php esc_attr_e('Title', 'news-communications-hub') ?>" />
                        </div>
                        <div class="col column-4 column-md-6">
                            <input type="text" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][link]" value="<?php echo esc_url($notification['link']); ?>" placeholder="<?php esc_attr_e('URL', 'news-communications-hub'); ?>" />
                        </div>
                        <div class="col column-2 column-md-6">
                            <input type="datetime-local" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][date]" value="<?php echo esc_attr($date_value); ?>" />
                        </div>
                        <div class="col column-2 column-md-6">
                            <input type="datetime-local" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][date_to]" value="<?php echo esc_attr($date_to_value); ?>" />
                        </div>

                        </div>

                        <div class="row">
                        <div class="col column-4 column-md-6">
                        <input type="color" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][color]" value="<?php echo esc_attr($color_value); ?>" style="height: 31px;display: inline-block;vertical-align: bottom; width: 20%;" />
                        <div style="width: calc(80% - 10px); display: inline-block;"><?php echo wp_kses( pcnch_display_icon_picker("pcnch_custom_notifications[{$index}][icon]", $icon_value), pcnch_kses_allowed_tags() ); ?></div>
                        </div>

                        <div class="col column-4 column-md-6">
                        <input type="text" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][tags]" value="<?php echo esc_attr($tags_value); ?>" placeholder="<?php esc_attr_e('Tags (comma separated)','news-communications-hub'); ?>" />
                        </div>
                        <div class="col column-4 column-md-12">
                        <label>
                            <input type="checkbox" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][show_on_first_visit]" value="1" <?php checked($show_on_first_visit_value, '1'); ?> />
                            <?php esc_attr_e('Show on first visit', 'news-communications-hub') ?>
                        </label>
                        &nbsp;
                        <label>
                            <input type="checkbox" name="pcnch_custom_notifications[<?php echo esc_attr($index); ?>][show_on_top]" value="1" <?php checked($show_on_top_value, '1'); ?> />
                            <?php esc_attr_e('Show on top', 'news-communications-hub') ?>
                        </label>
                        </div>
                        </div>
                        </div>
                        <div class="col column-2 column-md-1">
                        <button type="button" class="button nch-remove-notification" data-id="<?php echo esc_attr($notification['id']); ?>"><?php esc_attr_e('Remove', 'news-communications-hub') ?></button>
                        </div>
                    </div>
                </div>
            <?php }
        } else {
            echo '<em>' . esc_attr_e('No custom notifications', 'news-communications-hub') . '</em>';
        }
        ?>
    </div>
    </div>
    <br/>
    <button type="button" class="button" id="nch-add-notification"><?php esc_attr_e('Add Notification', 'news-communications-hub') ?></button>
    
    <?php

    $custom_notification_index = 0;
    $index = $custom_notification_index;

    $string_title = esc_attr__('Title', 'news-communications-hub');
    $string_url = esc_attr__('URL', 'news-communications-hub');
    $string_choose_icon = esc_attr__('Choose icon', 'news-communications-hub');
    $string_tags = esc_attr__('Tags', 'news-communications-hub');
    $string_on_first_visit = esc_attr__('Show on first visit', 'news-communications-hub');
    $string_on_top = esc_attr__('Show on top', 'news-communications-hub');
    $string_remove = esc_attr__('Remove', 'news-communications-hub');

    $iconPickerOutput = pcnch_display_icon_picker('pcnch_custom_notifications[__INDEX__][icon]', 'dashicons dashicons-book');
    $iconPickerOutputEscaped = str_replace('__INDEX__', '${index}', addslashes($iconPickerOutput));

    $new_notification_script = "
        jQuery(document).ready(function($) {
            var container = $('#nch-notifications-container');
            var addButton = $('#nch-add-notification');

            addButton.on('click', function() {
                var index = container.find('.nch-notification').length;
                var now = new Date();
                var nowLocal = now.toISOString().slice(0, 16);
                var newNotificationId = 'pcnch_' + Date.now().toString(36) + Math.random().toString(36).substring(2, 15);

                var newNotification = `
                    <div class=\"nch-notification new-notification\" id=\"` + newNotificationId + `\" style=\"margin-bottom: 15px; padding-bottom: 15px; border-bottom: 1px solid #ccc;\">
                        <input type=\"hidden\" name=\"pcnch_custom_notifications[` + index + `][id]\" value=\"` + newNotificationId + `\" />
                        <div class=\"row\">
                            <div class=\"col column-10 column-md-11\">
                                <div class=\"row\">
                                    <div class=\"col column-4 column-md-6\">
                                        <input type=\"text\" name=\"pcnch_custom_notifications[` + index + `][title]\" placeholder=\"{$string_title}\" />
                                    </div>
                                    <div class=\"col column-4 column-md-6\">
                                        <input type=\"text\" name=\"pcnch_custom_notifications[` + index + `][link]\" placeholder=\"{$string_url}\" />
                                    </div>
                                    <div class=\"col column-2 column-md-6\">
                                        <input type=\"datetime-local\" name=\"pcnch_custom_notifications[` + index + `][date]\" value=\"`+ nowLocal +`\" />
                                    </div>
                                    <div class=\"col column-2 column-md-6\">
                                        <input type=\"datetime-local\" name=\"pcnch_custom_notifications[` + index + `][date_to]\" placeholder=\"\" />
                                    </div>
                                </div>
                                <div class=\"row\">
                                    <div class=\"col column-4 column-md-6\">
                                        <input type=\"color\" name=\"pcnch_custom_notifications[` + index + `][color]\" value=\"#00a571\" style=\"height: 31px; display: inline-block; vertical-align: bottom; width: 20%;\" />
                                        <div style=\"width: calc(80% - 10px); display: inline-block;\">
                                            <div class=\"nch-icon-picker\">{$iconPickerOutputEscaped}</div>
                                        </div>
                                    </div>
                                    <div class=\"col column-4 column-md-6\">
                                        <input type=\"text\" name=\"pcnch_custom_notifications[` + index + `][tags]\" placeholder=\"{$string_tags}\" />
                                    </div>
                                    <div class=\"col column-4 column-md-12\">
                                        <label>
                                            <input type=\"checkbox\" name=\"pcnch_custom_notifications[` + index + `][show_on_first_visit]\" value=\"1\" />
                                            {$string_on_first_visit}
                                        </label>
                                        &nbsp;
                                        <label>
                                            <input type=\"checkbox\" name=\"pcnch_custom_notifications[` + index + `][show_on_top]\" value=\"1\" />
                                            {$string_on_top}
                                        </label>
                                    </div>
                                </div>
                            </div>
                            <div class=\"col column-2 column-md-1\">
                                <button type=\"button\" class=\"button nch-remove-notification\" data-id=\"` + newNotificationId + `\">{$string_remove}</button>
                            </div>
                        </div>
                    </div>
                `;

                container.append(newNotification);
                initializeIconPicker($('.new-notification'));

                // Toggle icon picker visibility on button click
                $('.nch-show-icon-picker').on('click', function() {
                    $(this).next('.nch-icon-picker').toggle();
                });
            });

            container.on('click', '.nch-remove-notification', function() {
                $(this).closest('.nch-notification').remove();
            });
        });
    ";

    wp_register_script('nch-new-notification', false, array('jquery'), '1.0.0', true);
    wp_enqueue_script('nch-new-notification');
    wp_add_inline_script('nch-new-notification', $new_notification_script);



}

/**
 * Saves custom notifications from the input to the database.
 *
 * @param array $input The input data for custom notifications.
 * @return array An array of sanitized and formatted notifications.
 */
function pcnch_save_custom_notifications($input) {
    $timezone_string = get_option('timezone_string', 'UTC');
    $timezone = new DateTimeZone($timezone_string);

    $notifications = array();
    if (isset($input) && is_array($input)) {
        foreach ($input as $notification) {
            if (isset($notification['id'], $notification['title'], $notification['link'], $notification['datetime'], $notification['icon'])) {
                $show_on_first_visit = isset($notification['show_on_first_visit']) && $notification['show_on_first_visit'] === '1' ? '1' : '0';
                $show_on_top = isset($notification['show_on_top']) && $notification['show_on_top'] === '1' ? '1' : '0';
                $icon = isset($notification['icon']) ? sanitize_text_field($notification['icon']) : 'eci pragmaticcoders-book-open';
                $color = isset($notification['color']) ? sanitize_text_field($notification['color']) : '#00a571';
                
                $date = new DateTime($notification['date'], $timezone);
                $date_to = !empty($notification['date_to']) ? (new DateTime($notification['date_to'], $timezone))->format('Y-m-d H:i:s') : null;

                if (isset($notification['tags']) && !empty($notification['tags'])) {
                    $tags = array_map('trim', explode(',', $notification['tags']));
                } else {
                    $tags = array();
                }

                $notifications[] = array(
                    'id' => sanitize_text_field($notification['id']),
                    'title' => sanitize_text_field($notification['title']),
                    'link' => esc_url_raw($notification['link']),
                    'date' => $date->format('Y-m-d H:i:s'),
                    'date_to' => $date_to,
                    'show_on_first_visit' => $show_on_first_visit,
                    'show_on_top' => $show_on_top,
                    'tags' => $tags,
                    'icon' => $icon,
                    'color' => $color
                );
            }
        }
    }
    return $notifications;
}

/**
 * Displays a dropdown for selecting the position of the notifications list.
 *
 * @return void
 */
function pcnch_list_position_callback() {
    $options = get_option('pcnch_list_position', 'right');
    ?>
    <select name="pcnch_list_position">
        <option value="left" <?php selected($options, 'left'); ?>><?php esc_attr_e('Left', 'news-communications-hub'); ?></option>
        <option value="right" <?php selected($options, 'right'); ?>><?php esc_attr_e('Right', 'news-communications-hub'); ?></option>
    </select>
    <p class="description"><?php esc_attr_e('Choose whether the notifications list is aligned to the left or right of the bell icon.', 'news-communications-hub'); ?></p>
    <?php
}

/**
 * Displays a number input field for setting the width of the notifications list.
 * 
 */
function pcnch_list_width_callback() {
    $width = get_option('pcnch_list_width', '320');
    ?>
    <input type="number" name="pcnch_list_width" min="200" value="<?php echo esc_attr($width); ?>" /> px
    <p class="description"><?php esc_attr_e('Set the width of the notifications list in pixels.', 'news-communications-hub'); ?></p>
    <?php
}

/**
 * Displays a form for managing icons for each public post type.
 *
 * @return void
 */
function pcnch_post_icons_callback() {
    $post_icons = get_option('pcnch_post_icons', array());
    $all_post_types = get_post_types(array('public' => true), 'objects');
    ?>
    <fieldset>
        <div class="admin-grid">
            <div class="row">
                <?php foreach ($all_post_types as $post_type): ?>
                    <div class="col column-4">
                        <label>
                            <?php echo esc_html($post_type->label); ?> <?php esc_attr_e('Icon','news-communications-hub'); ?>: <br>
                            <?php 
                                $selected_icon = $post_icons[$post_type->name] ?? 'dashicons dashicons-admin-post';
                                echo wp_kses( pcnch_display_icon_picker("pcnch_post_icons[{$post_type->name}]", $selected_icon), pcnch_kses_allowed_tags() );
                            ?>
                        </label>
                    </div>
                <?php endforeach; ?>
            </div>
        </div>
    </fieldset>
    <?php
}

/**
 * Renders the input field for specifying the lifetime of the data stored in localStorage.
 *
 * @return void
 */
function pcnch_localstorage_lifetime_callback() {
    $localstorage_lifetime = get_option('pcnch_localstorage_lifetime', 28);
    ?>
    <input type="number" name="pcnch_localstorage_lifetime" value="<?php echo esc_attr($localstorage_lifetime); ?>" min="1" />
    <p class="description"><?php esc_attr_e('Specify the lifetime of the data stored in localStorage in days.','news-communications-hub'); ?></p>
    <?php
}

/**
 * Renders the input field for specifying the lifetime of the database transient key.
 *
 * @return void
 */
function pcnch_transient_key_lifetime_callback() {
    $transient_key_lifetime = get_option('pcnch_transient_key_lifetime', 28);
    ?>
    <input type="number" name="pcnch_transient_key_lifetime" value="<?php echo esc_attr($transient_key_lifetime); ?>" min="1" />
    <p class="description"><?php esc_attr_e('Specify the lifetime of database transient key in days.','news-communications-hub'); ?></p>
    <?php
}

/**
 * Renders the input field for specifying the first visit lifetime of the data stored in localStorage.
 *
 * @return void
 */
function pcnch_localstorage_first_visit_lifetime_callback() {
    $localstorage_first_visit_lifetime = get_option('pcnch_localstorage_first_visit_lifetime', 1);
    ?>
    <input type="number" name="pcnch_localstorage_first_visit_lifetime" value="<?php echo esc_attr($localstorage_first_visit_lifetime); ?>" min="1" />
    <p class="description"><?php esc_attr_e('Specify the first visit lifetime of the data stored in localStorage in days.','news-communications-hub'); ?></p>
    <?php
}

/**
 * Renders the input field for specifying the first visit lifetime of the database transient key.
 *
 * @return void
 */
function pcnch_transient_key_first_visit_lifetime_callback() {
    $transient_key_first_visit_lifetime = get_option('pcnch_transient_key_first_visit_lifetime', 7);
    ?>
    <input type="number" name="pcnch_transient_key_first_visit_lifetime" value="<?php echo esc_attr($transient_key_first_visit_lifetime); ?>" min="1" />
    <p class="description"><?php esc_attr_e('Specify the first visit lifetime of database transient key in days.','news-communications-hub'); ?></p>
    <?php
}

/**
 * Renders the dropdown menu for selecting how often the first visit news CRON job should run.
 *
 * @return void
 */
function pcnch_cron_frequency_callback() {
    $cron_frequency = get_option('pcnch_cron_frequency', 'daily');
    $frequencies = array(
        'hourly' => 'Hourly',
        'twicedaily' => 'Twice Daily',
        'daily' => 'Daily',
        'every_two_days' => 'Every 2 Days',
        'weekly' => 'Weekly',
        'monthly' => 'Monthly'
    );
    ?>
    <select name="pcnch_cron_frequency">
        <?php foreach ($frequencies as $key => $label): ?>
            <option value="<?php echo esc_attr($key); ?>" <?php selected($cron_frequency, $key); ?>>
                <?php echo esc_html($label); ?>
            </option>
        <?php endforeach; ?>
    </select>
    <p class="description"><?php esc_attr_e('Select how often notifications CRON job should run.','news-communications-hub'); ?></p>
    <?php
}

function pcnch_cron_time_callback() {
    $cron_time = get_option('pcnch_cron_time', '09:00');
    ?>
    <input type="time" name="pcnch_cron_time" value="<?php echo esc_attr($cron_time); ?>" />
    <p class="description"><?php esc_attr_e('Select the time when the CRON job should run.', 'news-communications-hub'); ?></p>
    <?php
}

/**
 * Renders the checkbox for enabling or disabling logging.
 *
 * @return void
 */
function pcnch_enable_log_callback() {
    if ( ! function_exists( 'init_wp_filesystem' ) ) {
        require_once ABSPATH . 'wp-admin/includes/file.php';
    }
    WP_Filesystem();

    global $wp_filesystem;

    $enable_log = get_option('pcnch_enable_log', 0);

    $upload_dir = wp_upload_dir();
    $logs_dir = trailingslashit($upload_dir['basedir']) . 'pc-nch';
    $log_file = trailingslashit($logs_dir) . 'nch.txt';

    echo '<input type="checkbox" id="pcnch_enable_log" name="pcnch_enable_log" value="1" ' . checked(1, $enable_log, false) . '/>';
    echo '<label for="pcnch_enable_log"> ' . esc_html__('Enable log and display Log tab.', 'news-communications-hub') . '</label>';

    if ($enable_log) {
        if ( ! $wp_filesystem->is_dir($logs_dir) ) {
            wp_mkdir_p($logs_dir);
        }

        if ( ! $wp_filesystem->exists($log_file) ) {
            $wp_filesystem->put_contents($log_file, '', FS_CHMOD_FILE);
        }
    } else {
        if ( $wp_filesystem->exists($log_file) ) {
            $wp_filesystem->delete($log_file);
        }
    }
}

/**
 * Renders the icon picker for selecting the icon for read items.
 *
 * @return void
 */
function pcnch_read_icon_callback() {
    $value = get_option('pcnch_read_icon', 'dashicons dashicons-yes');
    $icon = $value ?? 'dashicons dashicons-yes';
    ?>

    <div class="nch-read-icon-wrapper">
        <div style="display: inline-block; min-width: 200px;">
            <?php 
            echo wp_kses( pcnch_display_icon_picker('pcnch_read_icon', $icon), pcnch_kses_allowed_tags() );
            ?>
        </div>
    </div>
    <?php
}

/**
 * Renders the icon picker and text input for marking all items as read.
 *
 * @return void
 */
function pcnch_mark_all_as_read_callback() {
    $value = get_option('pcnch_mark_all_as_read', '<i class="dashicons dashicons-yes"></i> <span>Mark all as read</span>');
    $icon = '';
    $text = '';

    if (preg_match('/<i class="(.+?)"><\/i>\s*<span>(.+?)<\/span>/', $value, $matches)) {
        $icon = trim($matches[1]);
        $text = trim($matches[2]);
    }

    ?>
    <div class="nch-icon-text-wrapper">
        <div style="display: inline-block; min-width: 200px;">
            <?php echo wp_kses( pcnch_display_icon_picker('pcnch_mark_all_as_read_icon', $icon), pcnch_kses_allowed_tags() ); ?>
        </div>
        <input type="text" class="nch-icon-text" value="<?php echo esc_attr($text); ?>" placeholder="Mark all as read">
        <input type="hidden" class="nch-icon-hidden" name="pcnch_mark_all_as_read" value="<?php echo esc_attr($value); ?>" data-initial-icon="<?php echo esc_attr($icon); ?>">
    </div>
    <?php
}

/**
 * Renders the icon picker and text input for showing read items.
 *
 * @return void
 */
function pcnch_show_read_items_callback() {
    $value = get_option('pcnch_show_read_items', '<i class="dashicons dashicons-visibility"></i> <span>Show read items</span>');
    $icon = '';
    $text = '';

    if (preg_match('/<i class="(.+?)"><\/i>\s*<span>(.+?)<\/span>/', $value, $matches)) {
        $icon = trim($matches[1]);
        $text = trim($matches[2]);
    }
    ?>
    <div class="nch-icon-text-wrapper">
        <div style="display: inline-block; min-width: 200px;">
            <?php echo wp_kses( pcnch_display_icon_picker('pcnch_show_read_items_icon', $icon), pcnch_kses_allowed_tags() ); ?>
        </div>
        <input type="text" class="nch-icon-text" value="<?php echo esc_attr($text); ?>" placeholder="Show read items">
        <input type="hidden" class="nch-icon-hidden" name="pcnch_show_read_items" value="<?php echo esc_attr($value); ?>" data-initial-icon="<?php echo esc_attr($icon); ?>">
    </div>
    <?php
}

/**
 * Renders the icon picker and text input for hiding read items.
 *
 * @return void
 */
function pcnch_hide_read_items_callback() {
    $value = get_option('pcnch_hide_read_items', '<i class="dashicons dashicons-visibility"></i> <span>Hide read items</span>');
    $icon = '';
    $text = '';

    if (preg_match('/<i class="(.+?)"><\/i>\s*<span>(.+?)<\/span>/', $value, $matches)) {
        $icon = trim($matches[1]);
        $text = trim($matches[2]);
    }
    ?>
    <div class="nch-icon-text-wrapper">
        <div style="display: inline-block; min-width: 200px;">
            <?php echo wp_kses( pcnch_display_icon_picker('pcnch_hide_read_items_icon', $icon), pcnch_kses_allowed_tags() ); ?>
        </div>
        <input type="text" class="nch-icon-text" value="<?php echo esc_attr($text); ?>" placeholder="Hide read items">
        <input type="hidden" class="nch-icon-hidden" name="pcnch_hide_read_items" value="<?php echo esc_attr($value); ?>" data-initial-icon="<?php echo esc_attr($icon); ?>">
    </div>
    <?php
}

/**
 * Renders the icon picker and text input for the "all read" message.
 *
 * @return void
 */
function pcnch_all_read_message_callback() {
    $value = get_option('pcnch_all_read_message', '<i class="dashicons dashicons-smiley"></i> <span>Congrats, you are up to date! Now you can impress your friends with your cutting-edge knowledge.</span>');
    $icon = '';
    $text = '';

    if (preg_match('/<i class="(.+?)"><\/i>\s*<span>(.+?)<\/span>/', $value, $matches)) {
        $icon = trim($matches[1]);
        $text = trim($matches[2]);
    }
    ?>
    <div class="nch-icon-text-wrapper">
        <div style="display: inline-block; min-width: 200px;">
            <?php echo wp_kses( pcnch_display_icon_picker('pcnch_all_read_message_icon', $icon), pcnch_kses_allowed_tags() ); ?>
        </div>
        <input type="text" class="nch-icon-text" value="<?php echo esc_attr($text); ?>" placeholder="Congrats, you are up to date!">
        <input type="hidden" class="nch-icon-hidden" name="pcnch_all_read_message" value="<?php echo esc_attr($value); ?>" data-initial-icon="<?php echo esc_attr($icon); ?>">
    </div>
    <?php
}

/**
 * Renders the checkbox for showing or hiding the Pragmatic Coders logo.
 *
 * @return void
 */
function pcnch_show_logo_callback() {
    $value = get_option('pcnch_show_logo', 1);
    echo '<input type="checkbox" name="pcnch_show_logo" value="1" ' . checked(1, $value, false) . ' /> ' . esc_html__('Show Pragmatic Coders Logo', 'news-communications-hub');
}

$options = [
    'pcnch_update_types',
    'pcnch_first_visit_display_limit',
    'pcnch_first_visit_post_types',
    'pcnch_localstorage_first_visit_lifetime',
    'pcnch_transient_key_first_visit_lifetime',
    'pcnch_post_icons',
    'pcnch_custom_notifications',
];

foreach ($options as $option) {
    add_action("update_option_{$option}", 'pcnch_generate_transients');
}

/**
 * Logs error messages to a log file within the uploads/pc-nch directory.
 *
 * This function creates the logs directory and log file if they do not exist,
 * and logs messages with the current timestamp.
 *
 * @param string $message The message to log.
 * @return void
 */
function pcnch_error_log($message) {
    // Get the WordPress upload directory
    $upload_dir = wp_upload_dir();
    $logs_dir = trailingslashit($upload_dir['basedir']) . 'pc-nch';
    $log_file = trailingslashit($logs_dir) . 'nch.txt';

    // Check if logging is enabled
    $is_log_enabled = get_option("pcnch_enable_log", 0);

    if ($is_log_enabled) {
        // Ensure the log directory exists
        if ( ! file_exists($logs_dir) ) {
            wp_mkdir_p($logs_dir); // Use WP_Filesystem to create the directory
        }

        // Initialize WP_Filesystem
        if ( ! function_exists('request_filesystem_credentials') ) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }

        $credentials = request_filesystem_credentials('', '', true, false);
        if ( ! WP_Filesystem($credentials) ) {
            return; // If unable to initialize WP_Filesystem, exit the function
        }

        // Check if the log file exists, and if not, create it
        global $wp_filesystem;
        if ( ! $wp_filesystem->exists($log_file) ) {
            $wp_filesystem->put_contents($log_file, '', FS_CHMOD_FILE); // Create an empty log file with write permissions
        }

        // Get the current time
        $current_time = current_time("mysql");

        // Prepare the log message
        $log_message = "[$current_time] $message" . PHP_EOL;

        // Use `WP_Filesystem` to append the log message to the file
        $current_contents = $wp_filesystem->get_contents($log_file); // Get the current contents of the file
        $new_contents = $current_contents . $log_message; // Append the new message

        // Write the new contents back to the file
        $wp_filesystem->put_contents($log_file, $new_contents, FS_CHMOD_FILE | LOCK_EX); // Overwrite the file with the new contents
    }
}


/**
 * Displays the contents of the log file, showing the last 1000 lines.
 *
 * @return void
 */
function pcnch_display_log() {
    $upload_dir = wp_upload_dir();
    $log_file = trailingslashit($upload_dir['basedir']) . 'pc-nch/nch.txt';

    if (file_exists($log_file)) {
        $lines = file($log_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        
        if (!empty($lines)) {
            $last_100_lines = array_slice($lines, -1000);
            $log_content = implode("\n", $last_100_lines);
        } else {
            $log_content = __('Log is empty.', 'news-communications-hub'); // Escaping moved to output
        }
    } else {
        $log_content = __('Log file not found.', 'news-communications-hub'); // Escaping moved to output
    }
    
    echo '<textarea id="nch-log" readonly rows="20" style="width:100%;">' . esc_textarea($log_content) . '</textarea>';
}

/**
 * Renders the "About" page for the Pragmatic Coders News Communications Hub.
 * 
 * @return void
 */
function pcnch_about_page() {
    ?>
    <h2>About Pragmatic Coders News Communications Hub <small>v <?php echo esc_html( PCNCH_VERSION ); ?></small></h2>

    <hr> 
    <h2>Overview</h2> 
    <p>The <strong>PragmaticCoders News Communications Hub</strong> is a flexible WordPress plugin designed to help you easily manage and display customizable notifications and news updates on your site. With robust admin options, Elementor icons integration, and flexible styling, this plugin provides all the tools you need to keep users informed and engaged. </p>
    <h2>Key Features</h2>
    <ul>
      <li>
        <strong>Custom Notifications</strong>: Add, edit, and manage notifications directly from the WordPress admin interface. You can display any type of post or page, including custom post types.
      </li>
      <li>
        <strong>Post Type Selection</strong>: Choose which post types (posts, pages, or custom types) you want to feature in your notifications.
      </li>
      <li>
        <strong>Custom Dates</strong>: Each custom notification can have specific start and end dates, allowing for time-sensitive announcements. This feature is ideal for promotions, events, or temporary updates.
      </li>
      <li>
        <strong>First-Time Visitor Notifications</strong>: Separate custom notifications into those that are visible for first-time visitors and those that appear for subsequent visits, enhancing user experience by presenting relevant information based on user behavior.
      </li>
      <li>
        <strong>Icon Support</strong>: Use Elementor's icon library or WordPress Dashicons to select icons for your notifications.
      </li>
      <li>
        <strong>Widgets and Shortcodes</strong>: Display notifications easily with the included widget or using the shortcode <code>[pcnch]</code>.
      </li>
      <li>
        <strong>Notification Log</strong>: Log important events, errors, and other data to a log file located in <code>/wp-content/uploads/pc-nch/nch.txt</code> for debugging and tracking purposes.
      </li>
      <li>
        <strong>Customizable Styling</strong>: Modify the look and feel of notifications via the CSS files located in the <code>assets/css</code> folder, or add your own custom styles.
      </li>
      <li>
        <strong>Optimized Code</strong>: The plugin is designed to minimize database queries and improve performance. Efficient queries and caching mechanisms are used to ensure optimal performance, reducing load times and server strain.
      </li>
    </ul>
    <h2>Usage</h2>
    <h3>Adding New Notifications</h3>
    <p>After activating the plugin, you can add new notifications through the <strong>News Communications Hub</strong> admin menu: </p>
    <ul>
      <li>
        <strong>Notification Title</strong>: Set the title that will appear in the notification bar.
      </li>
      <li>
        <strong>Post Type Selection</strong>: Choose which post types (e.g., posts, pages, or any custom types) should appear in the notification. You can select multiple post types.
      </li>
      <li>
        <strong>Custom Dates</strong>: Specify start and end dates for each notification, allowing for precise control over when notifications are visible to users.
      </li>
      <li>
        <strong>First-Time Visitor Options</strong>: Choose whether a notification should be visible only for first-time visitors or for all subsequent visits.
      </li>
      <li>
        <strong>Icons</strong>: Select an icon for your notification from the Elementor icon library or use the default Dashicons from WordPress.
      </li>
    </ul>
    <h3>Displaying Notifications</h3>
    <ol>
      <li>
        <strong>Shortcode</strong>: Use the <code>[pcnch]</code> shortcode to display the notification bar on any page or post.
      </li>
      <li>
        <strong>Widget</strong>: Alternatively, you can place the notification widget in any widgetized area of your theme (e.g., the sidebar or footer).
      </li>
      <li>
        <strong>Automatic Fetching</strong>: The plugin can automatically fetch the latest posts or custom post types and display them as notifications.
      </li>
    </ol>
    <h3>Managing Styles</h3>
    <p>All styles can be found in the <code>assets/css</code> folder. To modify the appearance of notifications, you can: </p>
    <ul>
      <li>Edit the provided CSS files.</li>
      <li>Override the styles in your theme by using custom CSS rules.</li>
    </ul>
    <h3>Notification Log</h3>
    <p>A log file is maintained at <code>/wp-content/uploads/pc-nch/nch.txt</code> to track important events such as errors, notification dispatches, and user interactions. This log can be useful for debugging issues or analyzing how notifications are being processed. </p>
    <h3>Optimized Database Queries</h3>
    <p>The plugin is built with performance in mind, utilizing optimized database queries to minimize the load on the server. By caching results when appropriate and reducing the number of queries executed per page load, the plugin ensures a seamless experience for users while conserving server resources.</p>
    <h2>Customization</h2>
    <ol>
      <li>
        <strong>Icons</strong>: Icons can be selected either from the Elementor icon library or WordPress Dashicons, allowing for rich visual customization.
      </li>
      <li>
        <strong>Post Type Inclusion</strong>: You can include any custom post types in the notifications, offering flexibility to display specific types of content.
      </li>
      <li>
        <strong>Shortcodes and Widgets</strong>: Whether through shortcodes or widgets, the plugin is designed to be easy to use and integrate with any WordPress theme.
      </li>
    </ol>
    <h2>FAQ</h2>
    <h4>Can I choose different post types for notifications?</h4>
    <p>Yes, you can select custom post types directly from the plugin settings to include posts, pages, or any other post type in your notifications.</p>
    <h4>Can I use icons from Elementor?</h4>
    <p>Yes, the plugin integrates with Elementor's icon library, allowing you to choose icons when creating notifications. You can also use WordPress's native Dashicons.</p>
    <h4>How do I add notifications to a page?</h4>
    <p>You can use the built-in widget or the provided shortcode <code>[pcnch]</code> to display notifications on any page. </p>
    <h4>Can custom notifications have specific start and end dates?</h4>
    <p>Yes, you can set specific start and end dates for each custom notification, allowing for time-sensitive announcements.</p>
    <h4>Is there a way to differentiate notifications for first-time visitors?</h4>
    <p>Yes, you can create notifications that are visible only for first-time visitors, as well as those that appear on subsequent visits.</p>
    <h4>How is the code optimized for database queries?</h4>
    <p>The plugin utilizes efficient database queries to minimize overhead, caching results when possible, and reducing the number of queries executed per page load for optimal performance.</p>
    <?php
}