<?php
/**
 * Plugin Name: News Communications Hub
 * Plugin URI: https://github.com/pragmaticcoders/WordPress-News-Communications-Hub
 * Description: Flexible WordPress plugin that allows you to display news notifications and updates on your website
 * Requires at least: 6.0
 * Requires PHP: 7.2
 * Version: 1.1.0
 * Author: Pragmatic Coders
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * Text Domain: news-communications-hub
 */

defined("ABSPATH") || exit();

define("PCNCH_VERSION", "1.1.0");

if (!defined("PCNCH_PLUGIN_URL")) {
    define("PCNCH_PLUGIN_URL", plugin_dir_url(__FILE__));
}

if (!defined("PCNCH_PLUGIN_PATH")) {
    define("PCNCH_PLUGIN_PATH", plugin_dir_path(__FILE__));
}

require_once PCNCH_PLUGIN_PATH . "admin/nch.php";
require_once PCNCH_PLUGIN_PATH . "includes/functions.php";

$pcnch_lifetime_localstorage = get_option("pcnch_localstorage_lifetime", 28);
$pcnch_lifetime_transient_key = get_option("pcnch_transient_key_lifetime", 28);
$pcnch_lifetime_localstorage_first_visit = get_option(
    "pcnch_localstorage_first_visit_lifetime",
    1
);
$pcnch_lifetime_transient_key_first_visit = get_option(
    "pcnch_transient_key_first_visit_lifetime",
    7
);

$pcnch_message_mark_all_as_read = get_option(
    "pcnch_mark_all_as_read",
    '<i class="dashicons dashicons-yes"></i> <span>Mark all as read</span>'
);
$pcnch_message_show_read_items = get_option(
    "pcnch_show_read_items",
    '<i class="dashicons dashicons-visibility"></i> <span>Show read items</span>'
);
$pcnch_message_hide_read_items = get_option(
    "pcnch_hide_read_items",
    '<i class="dashicons dashicons-visibility"></i> <span>Hide read items</span>'
);
$pcnch_message_all_read = get_option(
    "pcnch_all_read_message",
    '<i class="dashicons dashicons-smiley"></i> <span>Congrats, you are up to date!<span>'
);
$pcnch_icon_read = get_option(
    "pcnch_read_icon",
    'dashicons dashicons-yes'
);

$pcnch_show_logo = get_option("pcnch_show_logo", 1);

/**
 * Enqueues the JavaScript required for the plugin.
 *
 * This function adds the JavaScript file and localizes necessary data
 * for use within that script.
 *
 * @return void
 */
function pcnch_enqueue_scripts()
{
    global $pcnch_lifetime_localstorage,
        $pcnch_lifetime_transient_key,
        $pcnch_lifetime_localstorage_first_visit,
        $pcnch_lifetime_transient_key_first_visit;

    global $pcnch_message_mark_all_as_read, 
            $pcnch_message_show_read_items,
            $pcnch_message_hide_read_items,
            $pcnch_message_all_read,
            $pcnch_icon_read,
            $pcnch_show_logo;

    if (!wp_script_is("jquery", "enqueued")) {
        wp_enqueue_script("jquery");
    }

    wp_enqueue_script(
        "nch-script",
        PCNCH_PLUGIN_URL . "assets/js/nch-script.js",
        ["jquery"],
        "1.0.4",
        true
    );
    
    wp_localize_script("nch-script", "pcnch_data", [
        "localstorage_lifetime" => $pcnch_lifetime_localstorage,
        "localstorage_first_visit_lifetime" => $pcnch_lifetime_localstorage_first_visit,

        "message_mark_all_as_read" => $pcnch_message_mark_all_as_read,
        "message_show_read_items" => $pcnch_message_show_read_items,
        "message_hide_read_items" => $pcnch_message_hide_read_items,
        "message_all_read" => $pcnch_message_all_read,
        "icon_read" => $pcnch_icon_read,

        "show_logo" => $pcnch_show_logo,
        "plugin_url" => PCNCH_PLUGIN_URL,
    ]);

    wp_enqueue_style(
        "nch-style",
        PCNCH_PLUGIN_URL . "assets/css/nch-style.css",
        [],
        "1.0.1"
    );
}
add_action("wp_enqueue_scripts", "pcnch_enqueue_scripts");

/**
 * Adds attribute to a specified script to improve page load performance.
 *
 * @return string The modified script tag with the 'defer' or 'async' attribute added if the handle matches; original tag otherwise.
 */
add_filter( 'script_loader_tag', 'pcnch_script_load_attribute', 10, 2 );
function pcnch_script_load_attribute($tag, $handle) {
    if ( 'nch-script' === $handle ) {
        return str_replace( ' src', ' async="async" src', $tag );
    }
    return $tag;
}

/**
 * Fetches posts and notifications based on the visit status.
 * 
 * Checks if the request is from a first visit or a subsequent visit, then fetches the appropriate data.
 * Generates and returns data for posts and notifications via REST API response.
 * 
 * @param WP_REST_Request|null $request The request object. If null, it's considered a first visit.
 * 
 * @return WP_REST_Response The response containing success status, items, and creation timestamp.
 */
function pcnch_fetch_posts_and_notifications(WP_REST_Request $request = null) {
    // Determine if it's a first visit based on the 'pcnch_last_visit' parameter.
    $is_first_visit = is_null($request) || !$request->get_param("pcnch_last_visit");
    
    pcnch_error_log("------");
    pcnch_error_log("Is first visit: " . ($is_first_visit ? "Yes" : "No"));

    // Fetch appropriate data for the visit type (first or daily).
    $data = $is_first_visit 
        ? pcnch_generate_first_visit_transient() 
        : pcnch_generate_daily_transient();

    // Return the response with success status, items, and creation timestamp.
    return new WP_REST_Response(["success" => true, "items" => $data['items'], "created_at" => $data['created_at']], 200);
}

/**
 * Generates the transient data for the first visit.
 * 
 * Checks for existing transient data, and if not found, fetches fresh data for first-time visitors.
 * Caches the data in a transient for later use.
 * 
 * @return array The data for first visit notifications, including items and creation timestamp.
 */
function pcnch_generate_first_visit_transient() {
    global $pcnch_lifetime_transient_key_first_visit;

    // Define the transient key.
    $transient_key = "pcnch_first_visit_notifications";
    pcnch_error_log("Using transient: " . $transient_key);

    // Check if the transient data is already cached.
    $cached_data = get_transient($transient_key);
    if ($cached_data) {
        pcnch_error_log("Transient cache hit for key: " . $transient_key);
        return $cached_data;
    }

    // Fetch new data if no cached data exists.
    $data = pcnch_fetch_data(true);
    $created_at = current_time('Y-m-d H:i:s');
    
    // Prepare the data with timestamp.
    $data_with_timestamp = [
        'items' => $data,
        'created_at' => $created_at
    ];
    
    // Cache the data in a transient with the specified expiration time.
    set_transient($transient_key, $data_with_timestamp, $pcnch_lifetime_transient_key_first_visit * DAY_IN_SECONDS);
    pcnch_error_log("--- FETCHING END FOR " . $transient_key . " ---");

    return $data_with_timestamp;
}


/**
 * Generates the transient data for daily notifications.
 * 
 * Checks for existing transient data, and if not found, fetches fresh data for daily notifications.
 * Caches the data in a transient for later use.
 * 
 * @return array The data for daily notifications, including items and creation timestamp.
 */
function pcnch_generate_daily_transient() {
    global $pcnch_lifetime_transient_key;

    // Define the transient key.
    $transient_key = "pcnch_daily_notifications";
    pcnch_error_log("Using transient: " . $transient_key);

    // Check if the transient data is already cached.
    $cached_data = get_transient($transient_key);
    if ($cached_data) {
        pcnch_error_log("Transient cache hit for key: " . $transient_key);
        return $cached_data;
    }

    // Fetch new data if no cached data exists.
    $data = pcnch_fetch_data(false);
    $created_at = current_time('Y-m-d H:i:s');
    
    // Prepare the data with timestamp.
    $data_with_timestamp = [
        'items' => $data,
        'created_at' => $created_at
    ];
    
    // Cache the data in a transient with the specified expiration time.
    set_transient($transient_key, $data_with_timestamp, $pcnch_lifetime_transient_key * DAY_IN_SECONDS);
    pcnch_error_log("--- FETCHING END FOR " . $transient_key . " ---");

    return $data_with_timestamp;
}

/**
 * Generates both the first visit and daily transients.
 * 
 * This function is used to generate and cache the data for both first-time visits and daily notifications.
 */
function pcnch_generate_transients() {
    pcnch_generate_first_visit_transient();
    pcnch_generate_daily_transient();
}


/**
 * Fetches data for posts and notifications based on the visit type (first or subsequent).
 * 
 * This function retrieves the most recent posts and applies filters to notifications. It then combines the two datasets
 * and returns a limited number of items based on configuration.
 * 
 * @param bool $is_first_visit Indicates if the fetch is for the first visit or subsequent visits.
 * @return array An array of posts and notifications, sorted by date and configured settings.
 */
function pcnch_fetch_data($is_first_visit) {
    pcnch_error_log("--- FETCHING ---");

    // Get the display limit based on whether it's the first visit or not.
    $display_limit = $is_first_visit ? get_option("pcnch_first_visit_display_limit", 5) : get_option("pcnch_display_limit", 5);
    
    // Get the post types from the settings.
    $post_types = get_option("pcnch_post_types", ["post"]);
    pcnch_error_log("Post types: " . implode(", ", $post_types) . " | Display limit: " . $display_limit);

    // Prepare the query arguments for fetching posts.
    $args = [
        "post_type" => $post_types,
        "posts_per_page" => $display_limit,
        "post_status" => "publish",
        "orderby" => "date",
        "order" => "DESC",
    ];
    pcnch_error_log("WP_Query arguments: " . print_r($args, true));

    // Perform the WP_Query to fetch posts.
    $query = new WP_Query($args);
    $posts = [];
    
    // Iterate through the posts and prepare the data.
    while ($query->have_posts()) {
        $query->the_post();
        $posts[] = [
            "id" => get_the_ID(),
            "title" => get_the_title(),
            "link" => get_permalink(),
            "tag" => pcnch_get_main_tag(get_the_ID()),
            "post_type_name" => get_post_type_object(get_post_type())->labels->name,
            "date" => get_the_date("Y-m-d H:i:s"),
            "icon" => pcnch_get_post_icon(get_post_type()),
        ];
    }
    wp_reset_postdata();
    pcnch_error_log("Total posts fetched: " . count($posts));

    // Fetch the custom notifications from the settings.
    $notifications = get_option("pcnch_custom_notifications", []);
    pcnch_error_log("Fetched " . count($notifications) . " notifications.");

    $filtered_notifications = [];
    $now = new DateTime();

    // Process each notification based on the visit type.
    foreach ($notifications as $notification) {
        $notification["id"] = $notification["id"] ?? uniqid();
        $notification["date"] = $notification["date"] ?? current_time("Y-m-d\TH:i");
        $notification["link"] = $notification["link"] ?? "#";
        $notification["icon"] = $notification["icon"] ?? "dashicons dashicons-book";
        $notification["color"] = $notification["color"] ?? null;
        $notification["date_to"] = $notification["date_to"] ?? null;

        // Parse notification date and date_to
        $notification_date = DateTime::createFromFormat("Y-m-d\TH:i", $notification["date"]);
        if (!$notification_date) {
            pcnch_error_log("Invalid date for notification ID " . $notification["id"]);
            continue;
        }

        // If there is no date_to, set it as null
        $date_to = $notification["date_to"] ? DateTime::createFromFormat("Y-m-d\TH:i", $notification["date_to"]) : null;

        // Filter notifications for the first visit
        if ($is_first_visit) {
            if (!empty($notification["show_on_first_visit"])) {
                $filtered_notifications[] = $notification;
            }
        } else {
            // Filter for subsequent visits: considering notifications without an end date
            if (
                empty($notification["show_on_first_visit"]) &&
                $notification_date <= $now &&
                (!$date_to || $date_to >= $now)
            ) {
                $filtered_notifications[] = $notification;
                pcnch_error_log("Added notification ID: " . $notification["id"]);
            } else {
                pcnch_error_log("Skipped notification ID: " . $notification["id"]);
            }
        }
    }

    pcnch_error_log("Filtered notifications count: " . count($filtered_notifications));

    // Merge posts and notifications, then sort by date and "show_on_top" flag.
    $combined = array_merge($posts, $filtered_notifications);
    usort($combined, function ($a, $b) {
        if (!empty($a["show_on_top"]) && empty($b["show_on_top"])) {
            return -1;
        }
        if (empty($a["show_on_top"]) && !empty($b["show_on_top"])) {
            return 1;
        }
        return strtotime($b["date"]) - strtotime($a["date"]);
    });

    pcnch_error_log("Combined items count after sorting: " . count($combined));

    // Return the limited number of items.
    return array_slice($combined, 0, $display_limit);
}


/**
 * Schedules the cron job to refresh transients at a specific time and frequency.
 */
if (!wp_next_scheduled('pcnch_refresh_transients')) {
    // Get the scheduled time and frequency from the options
    $cron_frequency = get_option('pcnch_cron_frequency', 'daily');
    $cron_time = get_option('pcnch_cron_time', '09:00');

    // Get WordPress timezone
    $wp_timezone = wp_timezone();
    $current_date = new DateTime('now', $wp_timezone);
    $cron_datetime = DateTime::createFromFormat('H:i', $cron_time, $wp_timezone);
    
    if (!$cron_datetime) {
        $cron_datetime = new DateTime('09:00', $wp_timezone);
    }

    $cron_datetime->setDate(
        $current_date->format('Y'),
        $current_date->format('m'),
        $current_date->format('d')
    );

    if ($cron_datetime->getTimestamp() <= $current_date->getTimestamp()) {
        $cron_datetime->modify('+1 day');
    }

    // UTC convert
    $utc_timestamp = $cron_datetime->getTimestamp() - $wp_timezone->getOffset($cron_datetime);

    // Schedule the cron job
    wp_schedule_event($utc_timestamp, $cron_frequency, 'pcnch_refresh_transients');
    pcnch_error_log("Scheduled CRON job at {$cron_time} with frequency: {$cron_frequency} (in WordPress timezone)");
}


/**
 * Updates the cron schedule when the frequency or time option changes.
 */
add_action('update_option_pcnch_cron_frequency', 'pcnch_update_cron_schedule', 10, 2);
add_action('update_option_pcnch_cron_time', 'pcnch_update_cron_schedule', 10, 2);

function pcnch_update_cron_schedule() {
    // Clear the existing scheduled event
    if (wp_next_scheduled('pcnch_refresh_transients')) {
        wp_clear_scheduled_hook('pcnch_refresh_transients');
    }

    // Get the new frequency and time settings
    $cron_frequency = get_option('pcnch_cron_frequency', 'daily');
    $cron_time = get_option('pcnch_cron_time', '09:00');

    // Get WordPress timezone
    $wp_timezone = wp_timezone();
    $current_date = new DateTime('now', $wp_timezone);
    $cron_datetime = DateTime::createFromFormat('H:i', $cron_time, $wp_timezone);

    if (!$cron_datetime) {
        $cron_datetime = new DateTime('09:00', $wp_timezone);
    }

    $cron_datetime->setDate(
        $current_date->format('Y'),
        $current_date->format('m'),
        $current_date->format('d')
    );

    if ($cron_datetime->getTimestamp() <= $current_date->getTimestamp()) {
        $cron_datetime->modify('+1 day');
    }

    $utc_timestamp = $cron_datetime->getTimestamp() - $wp_timezone->getOffset($cron_datetime);

    // Reschedule the cron job
    wp_schedule_event($utc_timestamp, $cron_frequency, 'pcnch_refresh_transients');
    pcnch_error_log("Updated CRON schedule to {$cron_time}, frequency: {$cron_frequency}");
}


/**
 * Cron job callback function to refresh transients.
 */
add_action('pcnch_refresh_transients', function () {
    pcnch_error_log("--# Cron job triggered for transient refresh  #--");

    // Delete the existing transients and regenerate them
    delete_transient('pcnch_first_visit_notifications');
    delete_transient('pcnch_daily_notifications');
    pcnch_generate_transients();

    pcnch_error_log("--# Transient data refreshed #--");
});

/**
 * Clears transients when posts are saved, deleted, or custom notifications are updated.
 */
add_action('save_post', 'pcnch_clear_transients');
add_action('delete_post', 'pcnch_clear_transients');
add_action('update_option_pcnch_custom_notifications', 'pcnch_clear_transients');

/**
 * Clears transients and regenerates them.
 */
function pcnch_clear_transients() {
    pcnch_error_log("--# Clearing all transient data #--");

    // Delete the existing transients and regenerate them
    delete_transient('pcnch_first_visit_notifications');
    delete_transient('pcnch_daily_notifications');
    pcnch_generate_transients();
}

/**
 * Initializes the REST API route for fetching notifications.
 *
 * This function registers a new REST route that allows clients to
 * fetch notifications through a GET request.
 *
 * @return void
 */
add_action("rest_api_init", function () {
    register_rest_route("nch/v1", "/notifications", [
        "methods" => "GET",
        "callback" => "pcnch_fetch_posts_and_notifications",
        "permission_callback" => "__return_true",
    ]);
});


/**
 * Retrieves the main tag for a given post.
 *
 * This function fetches the main tag for a post based on its ID.
 * It checks if the post type is 'post' or 'blog', and returns the
 * main blog tag if available. Otherwise, it returns the first
 * term of the custom taxonomy associated with the post.
 *
 * @param int $post_id The ID of the post.
 * @return string|null The name of the main tag or null if not found.
 */
function pcnch_get_main_tag($post_id)
{
    $post_type = get_post_type($post_id);

    if ($post_type == "post" || $post_type == "blog") {
        $main_tag_id = get_post_meta($post_id, "main_blog_tag", true);
        return $main_tag_id
            ? get_term_by("id", $main_tag_id, "blog_tag")->name
            : null;
    }

    $terms = get_the_terms($post_id, $post_type . "_tag");
    return !is_wp_error($terms) && !empty($terms) ? $terms[0]->name : null;
}

/**
 * Retrieves the creation date of a transient.
 *
 * This function assumes that the transient stores its data as an array
 * with a `created_at` field containing the creation timestamp.
 *
 * @param string $transient_name The name of the transient.
 * @return string|null The creation date in 'Y-m-d H:i:s' format, or null if not found.
 */
function pcnch_get_transient_created_at($transient_name) {
    $transient_data = get_transient($transient_name);

    if ($transient_data && is_array($transient_data) && isset($transient_data['created_at'])) {
        return $transient_data['created_at'];
    }

    return null; // No creation date available
}

/**
 * Retrieves the icon associated with a specific post type.
 *
 * This function gets the icon class name for a given post type
 * from the options stored in the database. If no icon is found,
 * a default icon is returned.
 *
 * @param string $post_type The post type to retrieve the icon for.
 * @return string The icon class name.
 */
function pcnch_get_post_icon($post_type)
{
    $icons = get_option("pcnch_post_icons", []);
    return $icons[$post_type] ?? "dashicons dashicons-book";
}

/**
 * Generates the HTML output for the bell icon shortcode.
 *
 * @param array $atts Shortcode attributes, including 'color'.
 * @return string The HTML output for the shortcode.
 */
function pcnch_bell_icon_shortcode($atts)
{
    $atts = shortcode_atts(
        [
            "color" => "#00a571",
        ],
        $atts,
        "pcnch_bell_icon"
    );

    $color = esc_attr($atts["color"]);
    $list_position = get_option("pcnch_list_position", "left");
    $list_width = get_option("pcnch_list_width", "320");

    global $pcnch_message_mark_all_as_read, 
        $pcnch_message_show_read_items,
        $pcnch_message_hide_read_items,
        $pcnch_message_all_read;


    ob_start();
    ?>
    <div class="pragmaticcoders-nch">
        <div id="nch-bell-icon">
            <div id="nch-bell-button">
                <div class="icon" style="color: <?php echo esc_html( $color ); ?>;">
                    <?php include( PCNCH_PLUGIN_PATH . "assets/img/bell.svg" ); ?>
                </div>
                <span id="nch-notification-count" style="display: none;"></span>
            </div>
        </div>
        <div id="nch-notifications" style="display: none; min-width: <?php echo esc_html( $list_width ); ?>px; <?php echo $list_position === "left" ? "right" : "left"; ?>: -10px;">
            <div class="nch-info" style="display: none;"><?php echo wp_kses_post( $pcnch_message_all_read ); ?></div>
            <div class="notification-tools">
                <div class="action" id="mark-all-read"><?php echo wp_kses_post( $pcnch_message_mark_all_as_read ); ?></div>
                <div class="action" data-action="" id="show-hide"></div>
            </div>
        </div>
    </div>
    <?php return ob_get_clean();
}
add_shortcode("pcnch", "pcnch_bell_icon_shortcode");

/**
 * Registers the Pragmatic Coders NCH widget.
 *
 * This function registers a custom widget that displays the bell icon
 * and its associated notifications.
 *
 * @return void
 */
function pcnch_register_widget()
{
    register_widget("pcnch_Widget");
}
add_action("widgets_init", "pcnch_register_widget");

/**
 * Class representing the Pragmatic Coders widget.
 *
 * This class extends the WP_Widget class to create a custom widget
 * that shows the news communications hub bell icon.
 */
class pcnch_Widget extends WP_Widget
{
    /**
     * Constructor for the PragmaticCoders_pcnch_Widget class.
     *
     * This initializes the widget with its ID, name, and description.
     */
    function __construct()
    {
        parent::__construct(
            "pragmaticcoders_nch_widget",
            __("Pragmatic Coders NCH", "news-communications-hub"),
            [
                "description" => __(
                    "Show News Communications Hub bell icon",
                    "news-communications-hub"
                ),
            ]
        );
    }

    /**
     * Outputs the widget settings form in the admin panel.
     *
     * This function displays the form for configuring the widget's title
     * and color in the WordPress admin.
     *
     * @param array $instance Current settings for the widget.
     * @return void
     */
    public function form($instance)
    {
        $title = !empty($instance["title"]) ? $instance["title"] : "";
        $color = !empty($instance["color"]) ? $instance["color"] : "#00a571";
        ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( "title" ) ); ?>"><?php esc_html_e("Title:", "news-communications-hub"); ?></label>
            <input class="widefat" id="<?php echo esc_attr( $this->get_field_id( "title" ) ); ?>" name="<?php echo esc_attr( $this->get_field_name("title") ); ?>" type="text" value="<?php echo esc_attr($title); ?>">
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id("color") ); ?>"><?php esc_html_e("Color (hex):", "news-communications-hub"); ?></label>
            <input class="widefat" id="<?php echo esc_attr( $this->get_field_id("color") ); ?>" name="<?php echo esc_attr( $this->get_field_name("color") ); ?>" type="text" value="<?php echo esc_attr($color); ?>" placeholder="#ffffff">
        </p>
        <?php
    }

    /**
     * Processes the widget's options to be saved.
     *
     * This function sanitizes and saves the widget's title and color.
     *
     * @param array $new_instance New settings for the widget instance.
     * @param array $old_instance Old settings for the widget instance.
     * @return array Updated settings for the widget instance.
     */
    public function update($new_instance, $old_instance)
    {
        $instance = [];
        $instance["title"] = !empty($new_instance["title"])
            ? sanitize_text_field($new_instance["title"])
            : "";
        $instance["color"] = !empty($new_instance["color"])
            ? sanitize_text_field($new_instance["color"])
            : "#ffffff";
        return $instance;
    }

    /**
     * Outputs the content of the widget on the front end.
     *
     * This function displays the widget title and the bell icon
     * with the specified color.
     *
     * @param array $args Widget arguments.
     * @param array $instance Settings for the widget instance.
     * @return void
     */
    public function widget($args, $instance)
    {
        $title = apply_filters("widget_title", $instance["title"]);
        $color = !empty($instance["color"]) ? $instance["color"] : "#00a571";

        echo wp_kses_post( $args["before_widget"] );
        if (!empty($title)) {
            echo wp_kses_post( $args["before_title"] . $title . $args["after_title"] );
        }

        echo '<div class="nch-widget">';
        echo do_shortcode('[pcnch color="' . esc_attr($color) . '"]');
        echo "</div>";

        echo wp_kses_post( $args["after_widget"] );;
    }
}

/**
 * Displays a welcome message after the plugin is installed.
 *
 * @return void
 */
function pcnch_welcome_message()
{
    $logo_url = PCNCH_PLUGIN_URL . "assets/img/pragmaticcoders-logo.svg"; ?>
    <div class="notice notice-success is-dismissible">
        <div>
            <img src="<?php echo esc_url(
                $logo_url
            ); ?>" alt="Pragmatic Coders Logo" style="max-width: 32px; vertical-align: middle; margin-right: 10px;" />
            <h4 style="display: inline-block;"><?php esc_html_e(
                "Thank you for installing our plugin! We appreciate your support and hope you find it helpful.",
                "news-communications-hub"
            ); ?></h4>
        </div>
        <p>
            <?php esc_html_e("Get started by", "news-communications-hub"); ?>
            <a href="<?php echo esc_url( admin_url(
                "admin.php?page=pragmaticcoders-nch&tab=general"
            ) ); ?>"><?php esc_html_e("changing settings", "news-communications-hub"); ?></a>
            <?php esc_html_e("and", "news-communications-hub"); ?> 
            <a href="<?php echo esc_url( admin_url(
                "admin.php?page=pragmaticcoders-nch"
            ) ); ?>"><?php esc_html_e(
                    "adding your first notification",
                    "news-communications-hub"
                    ); ?></a>!
        </p>
       <?php if (pcnch_is_cache_plugin_active()) : ?>
        <hr>
        <p>
            <span style="color: black; font-weight: bold;">
                ⚠️ <?php esc_html_e(
                    "Potential issue detected",
                    "news-communications-hub"
                ); ?>: 
            </span>
            <span>
                <?php esc_html_e(
                    "The active caching plugin may interfere with News Communications Hub. Please make sure to exclude nch-script.js from JavaScript optimization in the cache settings.",
                    "news-communications-hub"
                ); ?>
            </span>
        </p>
        <?php endif; ?>

    </div>
    <?php
}

/**
 * Activation hook to set the welcome message option to true.
 * This ensures the welcome message is displayed after plugin activation.
 *
 * @return void
 */
function pcnch_activation_hook()
{
    update_option("pcnch_show_welcome_message", true);
}
register_activation_hook(__FILE__, "pcnch_activation_hook");

/**
 * Checks if the welcome message should be displayed.
 * If the welcome message option is set, it adds the admin notice to display it,
 * and then deletes the option to prevent the message from being shown again.
 *
 * @return void
 */
function pcnch_check_welcome_message()
{
    if (get_option("pcnch_show_welcome_message")) {
        add_action("admin_notices", "pcnch_welcome_message");
        delete_option("pcnch_show_welcome_message");
    }
}
add_action("admin_init", "pcnch_check_welcome_message");
